package cz.muni.fi.capella.converter.services;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Properties;
import java.util.stream.Collectors;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;

import cz.muni.fi.capella.converter.PropertyUtil;
import cz.muni.fi.capella.converter.ui.importing.LayoutViewAction;

/**
 * This class is responsible for loading properties from files.
 * 
 * @author Andrea Jonásová
 *
 */
public class ImportService {
	
	/**
	 * Gets all files of type <i>.properties</i> for given <code>filePath</code> starting with prefix <i>alg_</i>.
	 * <code>filePath</code> is directory located in this plugin, therefore special treatment is required.
	 * 
	 * @param filePath Given path to directory.
	 * @return List of all files represented as <code>LayoutViewAction</code>.
	 */
	public List<IContributionItem> getDefaultPropertyFiles(String filePath) {
		List<IContributionItem> files = Collections.emptyList();
		URL url = null;
		
		try {
			url = new URL(filePath);
		} catch (MalformedURLException e) {
			// should never happen
			e.printStackTrace();
		}
		
		if (url != null) {
			try {
				String directory = FileLocator.toFileURL(url).getFile();
				File dir = new File(directory);
				if (dir.isDirectory()) {
					files = getPropetyFiles(dir, true);
				}
			} catch (IOException e1) {
				// should never happen
				e1.printStackTrace();
			}
		}
		return files;
	}
	
	/**
	 * Gets all files of type <i>.properties</i> for given <code>filePath</code> starting with prefix <i>alg_</i>.
	 * 
	 * @param filePath Given path to directory.
	 * @return List of all files represented as <code>LayoutViewAction</code>.
	 */
	public List<IContributionItem> getCustomPropertyFiles(String filePath) {
		return getPropetyFiles(new File(filePath), false);
	}
	
	
	/**
	 * Filters files from given <code>dir</code> and maps them to <code>LayoutViewAction</code>.
	 * 
	 * @param dir Directory to get files from.
	 * @param areDefault <code>true</code> if dir is directory in this plugin <code>false</code> otherwise.
	 * @return List of all files represented as <code>LayoutViewAction</code>.
	 */
	private List<IContributionItem> getPropetyFiles(File dir, boolean areDefault) {
		return Arrays.asList(dir.list()).stream()
				.filter(file -> file.startsWith(PropertyUtil.PROPERTY_FILE_PREFIX) && file.endsWith(PropertyUtil.PROPERTY_FILE_SUFIX))
				.map(file -> new ActionContributionItem(new LayoutViewAction(file, areDefault)))
				.collect(Collectors.toList());
	}
	/**
	 * Load all properties from given file.
	 * 
	 * @param filePath Path to file from which properties should be loaded.
	 * @return Set of properties.
	 */
	public Properties loadCustomProperties(String filePath) {
		Properties properties = new Properties();
		
		try (InputStream inputStream = new FileInputStream(filePath)) {
			properties.load(inputStream);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return properties;
	}
	
	/**
	 * Load all properties from given file. 
	 * File represented by <code>filePath</code> is located in the plugin, therefore special treatment is required.
	 * 
	 * @param filePath Path to file from which properties should be loaded.
	 * @return Set of properties.
	 */
	public Properties loadDefProperties(String filePath) {
		Properties properties = new Properties();
		URL url = null;
		
		try {
			url = new URL(filePath);
		} catch (MalformedURLException e) {
			// should never happen
			e.printStackTrace();
		}
		
		if (url != null) {
			try (InputStream inputStream = new FileInputStream(FileLocator.toFileURL(url).getFile())){
				properties.load(inputStream);
			} catch (IOException e) {
				//should never happen
				e.printStackTrace();
			}
		}
		return properties;
	}
}
