package cz.muni.fi.capella.converter.services;

import java.util.Properties;

import javax.naming.OperationNotSupportedException;

import org.eclipse.elk.core.data.LayoutMetaDataService;
import org.eclipse.elk.core.data.LayoutOptionData;
import org.eclipse.elk.core.data.LayoutOptionData.Type;
import org.eclipse.elk.core.ui.views.LayoutPropertySource;
import org.eclipse.ui.views.properties.IPropertyDescriptor;

import cz.muni.fi.capella.converter.PropertyUtil;
import cz.muni.fi.capella.converter.exceptions.InvalidPropertyException;

/**
 * This class is responsible for manipulation with properties.
 * 
 * @author Andrea Jonásová
 *
 */
public class PropertyService {

	private final LayoutPropertySource store = PropertyUtil.getPropertyStore();
	
	/**
	 * Loads properties from layout property store.
	 * 
	 * @return Loaded properties.
	 * @throws OperationNotSupportedException when <code>store</code> is <code>null</code>.
	 */
	public Properties loadPropertiesFromPropertySource() throws OperationNotSupportedException {
		Properties properties = new Properties();
		if (store == null) {
			throw new OperationNotSupportedException("Could not find any property source.");
		}
		
		// load all pairs (id, value) from layout property source
		for (IPropertyDescriptor descriptor : store.getPropertyDescriptors()) {
			
			String id = (String) descriptor.getId();
			Object value = store.getPropertyValue(id);
			
			final LayoutOptionData optionData = LayoutMetaDataService.getInstance().getOptionData(id);
			
			String translatedValue = store.translateFromUI(value, optionData).toString();
			properties.setProperty(id, translatedValue);
		}
		return properties;
	}
	
	
	/**
	 * Tries to convert properties to how they are stored in layout property source.
	 * 
	 * @param properties Properties to convert.
	 * @return Converted properties.
	 * @throws InvalidPropertyException if any of the properties cannot be converted.
	 * @throws OperationNotSupportedException when <code>store</code> is <code>null</code>.
	 */
	public Properties tryConvertProperties(Properties properties) throws InvalidPropertyException, 
			OperationNotSupportedException {
		
		if (store == null) {
			throw new OperationNotSupportedException("Could not find any property source.");
		}
		
		Properties convertedProperties = properties;
		
		for (var entry : properties.entrySet()) {
			
			String id = (String) entry.getKey();
			Object value = entry.getValue();
			final LayoutOptionData optionData = LayoutMetaDataService.getInstance().getOptionData(id);
		
			// check if id is valid
			if (optionData == null) {
				throw new InvalidPropertyException(entry.toString());
			}
			
			// check if value is valid
			Object parsedValue = optionData.parseValue(value.toString());
			if (parsedValue == null) {
				throw new InvalidPropertyException(entry.toString());
			}
			
			// additional conversion for enum, because translateToUI can't process enum sets correctly
			if (Type.ENUMSET.equals(optionData.getType())) {
				value = parsedValue;
			}
			value = store.translateToUI(value, optionData);
			convertedProperties.put(id, value);
		}
		return convertedProperties;
	}
	
	/**
	 * Writes properties into layout property source.
	 * 
	 * @param properties Properties to write.
	 * @throws OperationNotSupportedException when <code>store</code> is <code>null</code>.
	 */
	public void writePropertiesToPropertySource(Properties properties) throws OperationNotSupportedException {
		if (store == null) {
			throw new OperationNotSupportedException("Could not find any property source.");
		}
		for (var property : properties.entrySet()) {
			store.setPropertyValue(property.getKey(), property.getValue());
		}
	}
}
